@@ -10,12 +10,10 @@ urls.py - Mapping of URL's to our various views. Note we always used NAMED
 from django.conf import settings
 from django.conf.urls.defaults import *
 from django.contrib.auth.decorators import login_required
-from django.contrib.syndication.views import feed as django_feed
 
 from helpdesk import settings as helpdesk_settings
 from helpdesk.views import feeds
 
-
 urlpatterns = patterns('helpdesk.views.staff',
     url(r'^dashboard/$',
         'dashboard',
@@ -41,6 +39,10 @@ urlpatterns = patterns('helpdesk.views.staff',
         'followup_edit',
         name='helpdesk_followup_edit'),
 
+    url(r'^tickets/(?P<ticket_id>[0-9]+)/followup_delete/(?P<followup_id>[0-9]+)/$',
+        'followup_delete',
+        name='helpdesk_followup_delete'),
+
     url(r'^tickets/(?P<ticket_id>[0-9]+)/edit/$',
         'edit_ticket',
         name='helpdesk_edit'),
@@ -141,11 +143,11 @@ urlpatterns += patterns('helpdesk.views.public',
 )
 
 urlpatterns += patterns('',
-    url(r'^rss/user/(?P<user_name>[A-Za-z0-9_-]+)/$',
+    url(r'^rss/user/(?P<user_name>[\.A-Za-z0-9_-]+)/$',
         login_required(feeds.OpenTicketsByUser()),
         name='helpdesk_rss_user'),
     
-    url(r'^rss/user/(?P<user_name>[A-Za-z0-9_-]+)/(?P<queue_slug>[A-Za-z0-9_-]+)/$',
+    url(r'^rss/user/(?P<user_name>[\.A-Za-z0-9_-]+)/(?P<queue_slug>[A-Za-z0-9_-]+)/$',
         login_required(feeds.OpenTicketsByUser()),
         name='helpdesk_rss_user_queue'),
     
